-- MultiQuestTracker (MQT) by Zerorez
-- Modified by Blackstorm / 20140409
MQT = {}
local LMP = LibStub:GetLibrary("LibMediaProvider-1.0")
local LAM = LibStub:GetLibrary("LibAddonMenu-1.0")
local fontList = LMP:List('font')
local fontStyles = {"normal", "outline", "thick-outline", "shadow", "soft-shadow-thick", "soft-shadow-thin"}

local bt = {}
local function MQT_DelayBuffer(key, buffer)
	if key == nil then return; end
	if bt[key] == nil then bt[key] = {} end
	bt[key].buffer = buffer or 3
	bt[key].now = GetFrameTimeMilliseconds()
	if bt[key].last == nil then bt[key].last = bt[key].now end
	bt[key].diff = bt[key].now - bt[key].last
	bt[key].eval = bt[key].diff >= bt[key].buffer
	if bt[key].eval then bt[key].last = bt[key].now end
	return bt[key].eval
end

-- chain caller
local function Chain(obj)
	local T={}
	setmetatable(T,{__index = function(self,fun)
		if fun=="__END" then
			return obj
		end
		return function(self,...)
			assert(obj[fun],fun.." missing in object")
			obj[fun](obj,...)
			return self
		end
	end})
	return T
end

local WM = WINDOW_MANAGER

-- create background for MQT
MQTbg = Chain(WM:CreateControl("MQT_bg",ZO_QuestTracker,CT_STATUSBAR))
	:SetOrientation(2)
	:SetBarAlignment(1)
	:SetDimensions(242,100)
	:SetSimpleAnchorParent(0,0)
	:SetGradientColors(0,0,0,.5,0,0,0,0)
	:SetDrawLayer(1)
	:SetMouseEnabled(true)
	:SetMovable(true)
.__END

-- set proper draw layers for quest tracker
ZO_QuestTrackerContainer:SetSimpleAnchor(MQT_bg,20,15)
ZO_QuestTrackerContainer:SetDrawLayer(2)
ZO_QuestTrackerTimerAnchor:SetDrawLayer(2)


-------------------------------------------------------
-- Updates
-------------------------------------------------------
function MQTUpdate()
	MQT_HideCheck()
	MQT_UpdateText()
end

function MQT_HideCheck()
	if not MQT_DelayBuffer("HideCheck", 2) then return; end
	local menuHidden = ZO_KeybindStripControlBackground:IsHidden()
	local interactHidden = ZO_InteractWindow:IsHidden()
	local journalHidden = ZO_QuestJournal:IsHidden()
	if (menuHidden == true) and (interactHidden == true) then
		ZO_QuestTracker:SetHidden(false)
		ZO_FocusedQuestTrackerPanel:SetHidden(true)
		ZO_QuestTrackerContainer:SetHidden(false)
		ZO_QuestTrackerTimerAnchor:SetHidden(false)
		ZO_QuestTracker:SetMovable(true)
		ZO_QuestTracker:SetMouseEnabled(true)
	elseif (menuHidden == false) or (interactHidden == false) then
		ZO_QuestTracker:SetHidden(true)
	end
	if journalHidden == false then
		MQTOnQuestJournalOpen()
	elseif journalHidden == true then
	end	
end

function MQT_UpdateText()
	if not MQT_DelayBuffer("TextUpdate", 2) then return; end
	MQT_ReloadText()
end


--------------------------------------------------
-- Quest Journal
--------------------------------------------------

function MQTToggleTracked()
	local cbControl = WM:GetMouseOverControl()
	local pControl = cbControl:GetParent()
	local qindex = pControl["questIndex"]
	local tControl = ZO_QuestTracker["tracker"]
	if GetIsTracked(1,qindex,0) == true then
		SetTracked(1,false,qindex,0)
		tControl:InitialTrackingUpdate()
		MQTTrackingTable()
	else
		SetTracked(1,true,qindex,0)
		tControl:InitialTrackingUpdate()
		MQTTrackingTable()
	end
end

function MQTGetQuestControls()
	MQT.qtable = {}
	qtindex = 1
	if GetNumJournalQuests == 0 then return; end
	for i=1, 30 do
		if QUEST_JOURNAL.navigationTree.rootNode.children ~= nil then
			if QUEST_JOURNAL.navigationTree.rootNode.children[i] ~= nil then
				qcontainer = QUEST_JOURNAL.navigationTree.rootNode.children[i]
				for node=1, 30 do
					if qcontainer.children[node] ~= nil then
						MQT.qtable[qtindex] = {}
						MQT.qtable[qtindex].control = qcontainer.children[node].control
						MQT.qtable[qtindex].data = qcontainer.children[node].data
						qtindex = qtindex + 1
					end
				end
			end
		end
	end
end

-- runs when quest journal is open
function MQTOnQuestJournalOpen()
	MQTGetQuestControls()
	for i=1, 30 do
		if MQT.qtable[i] ~= nil then
			qindex = MQT.qtable[i].data.questIndex
			qcontrol = MQT.qtable[i].control
			qtracked = GetIsTracked(1,qindex,0)
			qname = qcontrol:GetName()
			MQTCreateCheckbox(qcontrol, qtracked, qname)
		end
	end
end

-- create check boxes in journal
function MQTCreateCheckbox(qcontrol, qtracked, qname)
	if qtracked == true then
		cbState = BSTATE_PRESSED
	else
		cbState = BSTATE_NORMAL
	end

	local cname = tostring( qname .. "MQT_Checkbox" )
	if qcontrol:GetNamedChild("MQT_Checkbox") ~= nil then return; end
	
	MQTcb = Chain(WM:CreateControlFromVirtual(cname,qcontrol,"ZO_CheckButton"))
		:SetAnchor(RIGHT,qcontrol,LEFT,-25,0)
		:SetDimensions(15,15)
		:SetHidden(false)
		:SetHandler("OnMouseUp", MQTToggleTracked)
		:SetState(cbState, qtracked)
	.__END
	
end

------------------------------------------------------
-- Quest Tables
------------------------------------------------------
function MQTTrackingTable()
	MQT.Tracked = {}
	for i=1, MAX_JOURNAL_QUESTS do
		if IsValidQuestIndex(i) then
			MQT.Tracked[i] = GetIsTracked(1,i,0)
		end
	end
end

function MQT_GetTrackerEntries()
	local qControls = {}
	qControls.objectives = {}
	qControls.headers = {}
	qControls.other = {}
	
	for i,v in pairs(ZO_QuestTracker.tracker.headerPool.m_Active) do
		table.insert(qControls.headers, v)
	end
	
	for i,v in pairs(ZO_QuestTracker.tracker.conditionPool.m_Active) do
		table.insert(qControls.objectives, v)
	end

	return qControls
end

-------------------------------------------------------
-- tracker reloading functions
--------------------------------------------------------
function MQT_ReloadText()
	local qControls = MQT_GetTrackerEntries()
	
	local MyheaderFont = MQT.Defaults.headerFont
	local MyheaderSize = MQT.Defaults.headerSize
	local MyheaderStyle = MQT.Defaults.headerStyle
	local MyheaderColor = {}
	
	if MQT.SV.headerFont ~= nil then
		MyheaderFont = MQT.SV.headerFont
	end
	if MQT.SV.headerSize ~= nil then
		MyheaderSize = MQT.SV.headerSize
	end
	if MQT.SV.headerStyle ~= nil then
		MyheaderStyle = MQT.SV.headerStyle
	end
	if MQT.SV.headerColor ~= nil then
		MyheaderColor.r = MQT.SV.headerColor.r
		MyheaderColor.g = MQT.SV.headerColor.g
		MyheaderColor.b = MQT.SV.headerColor.b
		MyheaderColor.a = MQT.SV.headerColor.a
	else
		MyheaderColor.r = MQT.Defaults.headerColor.r
		MyheaderColor.g = MQT.Defaults.headerColor.g
		MyheaderColor.b = MQT.Defaults.headerColor.b
		MyheaderColor.a = MQT.Defaults.headerColor.a		
	end

	local MyqTextFont = MQT.Defaults.qTextFont
	local MyqTextSize = MQT.Defaults.qTextSize
	local MyqTextStyle = MQT.Defaults.qTextStyle
	local MyqTextColor = {}
	
	if MQT.SV.qTextFont ~= nil then
		MyqTextFont = MQT.SV.qTextFont
	end
	if MQT.SV.qTextSize ~= nil then
		MyqTextSize = MQT.SV.qTextSize
	end
	if MQT.SV.qTextStyle ~= nil then
		MyqTextStyle = MQT.SV.qTextStyle
	end
	if MQT.SV.qTextColor ~= nil then
		MyqTextColor.r = MQT.SV.qTextColor.r
		MyqTextColor.g = MQT.SV.qTextColor.g
		MyqTextColor.b = MQT.SV.qTextColor.b
		MyqTextColor.a = MQT.SV.qTextColor.a
	else
		MyqTextColor.r = MQT.Defaults.qTextColor.r
		MyqTextColor.g = MQT.Defaults.qTextColor.g
		MyqTextColor.b = MQT.Defaults.qTextColor.b
		MyqTextColor.a = MQT.Defaults.qTextColor.a		
	end
		for i,v in pairs(qControls.headers) do
				v:SetFont(("%s|%s|%s"):format(LMP:Fetch('font', MyheaderFont), MyheaderSize, MyheaderStyle))
				v:SetColor(MyheaderColor.r, MyheaderColor.g, MyheaderColor.b, MyheaderColor.a)
	
		end
	
		for i,v in pairs(qControls.objectives) do
				v:SetFont(("%s|%s|%s"):format(LMP:Fetch('font', MyqTextFont), MyqTextSize, MyqTextStyle))
				v:SetColor(MyqTextColor.r, MyqTextColor.g, MyqTextColor.b, MyqTextColor.a)
	
		end
end

-------------------------------------------------------
-- Options
-------------------------------------------------------
function MQT_bgGetAlpha()
	local alpha = MQT_bg:GetAlpha()
	local mapAlpha = alpha * 100
	return mapAlpha
end

function MQT_bgSetAlpha(newAlpha)
	local newAlphaPercent = newAlpha / 100
	MQT_bg:SetAlpha(newAlphaPercent)
	MQT.SV.bgAlpha = newAlpha
end


---------------------------------------
function MQT_GetHeaderFont()
	return MQT.SV.headerFont
end

function MQT_SetHeaderFont(newFont)
	MQT.SV.headerFont = newFont
	MQT_ReloadText()
end


function MQT_GetHeaderStyle()
	return MQT.SV.headerStyle
end

function MQT_SetHeaderStyle(newStyle)
	MQT.SV.headerStyle = newStyle
	MQT_ReloadText()
end


function MQT_GetHeaderSize()
	return MQT.SV.headerSize
end

function MQT_SetHeaderSize(newSize)
	MQT.SV.headerSize = newSize
	MQT_ReloadText()
end


function MQT_GetHeaderColor()
	return MQT.SV.headerColor.r, MQT.SV.headerColor.g, MQT.SV.headerColor.b, MQT.SV.headerColor.a
end

function MQT_SetHeaderColor(r,g,b,a)
	MQT.SV.headerColor.r = r
	MQT.SV.headerColor.g = g
	MQT.SV.headerColor.b = b
	MQT.SV.headerColor.a = a
	MQT_ReloadText()
end


-------------------------------------
function MQT_GetQTextFont()
	return MQT.SV.qTextFont
end

function MQT_SetQTextFont(newFont)
	MQT.SV.qTextFont = newFont
	MQT_ReloadText()
end


function MQT_GetQTextStyle()
	return MQT.SV.qTextStyle
end

function MQT_SetQTextStyle(newStyle)
	MQT.SV.qTextStyle = newStyle
	MQT_ReloadText()
end


function MQT_GetQTextSize()
	return MQT.SV.qTextSize
end

function MQT_SetQTextSize(newSize)
	MQT.SV.qTextSize = newSize
	MQT_ReloadText()
end


function MQT_GetQTextColor()
	return MQT.SV.qTextColor.r, MQT.SV.qTextColor.g, MQT.SV.qTextColor.b, MQT.SV.qTextColor.a
end

function MQT_SetQTextColor(r,g,b,a)
	MQT.SV.qTextColor.r = r
	MQT.SV.qTextColor.g = g
	MQT.SV.qTextColor.b = b
	MQT.SV.qTextColor.a = a
	MQT_ReloadText()
end



----------------------------------------------------------
-- On Addon Load
----------------------------------------------------------
function MQTLoadVars()
	if MQT.SV.Tracked ~= nil then
		for k, v in pairs(MQT.SV.Tracked) do
			SetTracked(1,v,k,0)
		end
	end
	
	if MQT.SV.position ~= nil then
		MQT_bg:ClearAnchors()
		MQT_bg:SetAnchor(MQT.SV.position.point, GuiRoot, MQT.SV.position.relativePoint, MQT.SV.position.offsetX, MQT.SV.position.offsetY)
		MQT_bg:SetMovable(true)
	end
	
	MQT_ReloadText()
end

-- OnInit...
function MQTOnInit(eventCode, addOnName)
	if addOnName ~= "MultiQuestTracker" then return; end
	
	MQT.Defaults =  {   ["position"] = { ["point"]=TOPLEFT, ["relativePoint"]=TOPLEFT, ["offsetX"]=1200, ["offsetY"]=200 },
						["bgAlpha"] = 100,
						["headerFont"] = "Univers 67",
						["headerStyle"] = "soft-shadow-thin",
						["headerSize"] = 18,
						["headerColor"] = { ["r"]=.8, ["g"]=.5, ["b"]=0, ["a"]=1 },
						["qTextFont"] = "Univers 67",
						["qTextStyle"] = "soft-shadow-thin",
						["qTextSize"] = 17,
						["qTextColor"] = { ["r"]=1, ["g"]=1, ["b"]=1, ["a"]=1 },
						["trackerWidth"] = 275,
					}
	MQT.SV = ZO_SavedVars:New( "MQT" , 5 , nil , MQT.Defaults , nil )
	
	
	MQT_bg:SetHandler("OnMouseUp", function(self) 
		MQT.SV.position.offsetX = MQT_bg:GetLeft()
		MQT.SV.position.offsetY = MQT_bg:GetTop()
		end)
	
	if MQT.SV ~= nil then
		MQTLoadVars()
	end
	
	MQTpanel = LAM:CreateControlPanel("MQT_OptionsPanel", "MQT Settings")
		LAM:AddHeader(MQTpanel, "MQT_bgHeader", "Background")	
			LAM:AddSlider(MQTpanel, "MQT_Option_BGAlpha", "Background Alpha", "Change the transparency of the background", 0, 100,1, MQT_bgGetAlpha, MQT_bgSetAlpha)
		LAM:AddHeader(MQTpanel, "MQT_HeaderHeader", "Quest Names")
			LAM:AddDropdown(MQTpanel, "MQT_Option_HeaderFont", "Name Font", "Set the font for the quest names", fontList, MQT_GetHeaderFont, MQT_SetHeaderFont)
			LAM:AddDropdown(MQTpanel, "MQT_Option_HeaderStyle", "Name Font Style", "Set the font style the quest names", fontStyles, MQT_GetHeaderStyle, MQT_SetHeaderStyle)
			LAM:AddSlider(MQTpanel, "MQT_Option_HeaderSize", "Name Font Size", "Change the font size for quest names", 8, 45,1, MQT_GetHeaderSize, MQT_SetHeaderSize)
			LAM:AddColorPicker(MQTpanel, "MQT_Option_HeaderColor", "Name Font Color", "Change the color of quest names", MQT_GetHeaderColor, MQT_SetHeaderColor)
		LAM:AddHeader(MQTpanel, "MQT_QTextHeader", "Objective Text")
			LAM:AddDropdown(MQTpanel, "MQT_Option_QTextFont", "Objective Font", "Set the font for the quest text", fontList, MQT_GetQTextFont, MQT_SetQTextFont)
			LAM:AddDropdown(MQTpanel, "MQT_Option_QTextStyle", "Objective Font Style", "Set the font style the quest text", fontStyles, MQT_GetQTextStyle, MQT_SetQTextStyle)
			LAM:AddSlider(MQTpanel, "MQT_Option_QTextSize", "Objective Font Size", "Change the font size for quest text", 8, 45,1, MQT_GetQTextSize, MQT_SetQTextSize)
			LAM:AddColorPicker(MQTpanel, "MQT_Option_QTextrColor", "Objective Font Color", "Change the color of quest text", MQT_GetQTextColor, MQT_SetQTextColor)
end

EVENT_MANAGER:RegisterForEvent( "MQT", EVENT_ADD_ON_LOADED , MQTOnInit )

